@@ -104,6 +104,9 @@ namespace traits {
                 
                 // Making space for the elements
                 res.mem_resize(static_cast<unsigned>(x.size()));
+
+                // In order to access the internal arrays of the SpMat class
+                res.sync();
                 
                 // Copying elements
                 std::copy(i.begin(), i.end(), arma::access::rwp(res.row_indices));
@@ -119,6 +122,9 @@ namespace traits {
                 // Making space for the elements
                 res.mem_resize(static_cast<unsigned>(x.size()));
                 
+                // In order to access the internal arrays of the SpMat class
+                res.sync();
+                
                 // Copying elements
                 std::copy(i.begin(), i.end(), arma::access::rwp(res.row_indices));
                 std::copy(p.begin(), p.end(), arma::access::rwp(res.col_ptrs));
@@ -137,6 +143,9 @@ namespace traits {
                 // Making space for the elements
                 res.mem_resize(static_cast<unsigned>(x.size()));
                 
+                // In order to access the internal arrays of the SpMat class
+                res.sync();
+                
                 // Copying elements
                 std::copy(i.begin(), i.end(), arma::access::rwp(res.row_indices));
                 std::copy(p.begin(), p.end(), arma::access::rwp(res.col_ptrs));
@@ -149,172 +158,29 @@ namespace traits {
                 }
             }
             else if (type == "dgTMatrix" || mat.is("dgTMatrix")) {
-                IntegerVector ti = mat.slot("i");
-                IntegerVector tj = mat.slot("j");
-                Vector<RTYPE> tx = mat.slot("x");
+                arma::urowvec ti = mat.slot("i");
+                arma::urowvec tj = mat.slot("j");
+                arma::Col<T> tx = mat.slot("x");             
                 
-                // Create a map to sort the i, j, x
-                typedef std::pair<int, int> Key;
-                typedef std::map<Key, double> Map;
-                Map jix;
-                int tnnz = tx.size();
-                for(int idx = 0; idx < tnnz; idx++){
-                    Key p(tj[idx], ti[idx]);
-                    if (jix.find(p) != jix.end()) {
-                        jix[p] += tx[idx];
-                    } else {
-                        jix[p] = tx[idx];
-                    }
-                }
-                
-                // Get the new i, j, x;
-                int nnz = jix.size();
-                std::vector<int> i;
-                std::vector<int> j;
-                std::vector<double> x;
-                IntegerVector p = IntegerVector(ncol + 1);
-                
-                for(Map::iterator tmp = jix.begin(); tmp != jix.end(); tmp++){
-                    j.push_back((tmp -> first).first);
-                    i.push_back((tmp -> first).second);
-                    x.push_back(tmp -> second);
-                }
-                
-                // Count the number of nnz in each column
-                for(int idx = 0; idx < nnz; idx++){
-                    int col = j[idx];
-                    p[col + 1]++;
-                }
-                
-                // Cumsum p
-                for(int col = 0, cumsum = 0; col < ncol + 1; col++){
-                    cumsum += p[col];
-                    p[col] = cumsum;
-                }
-                
-                // Making space for the elements
-                res.mem_resize(static_cast<unsigned>(x.size()));
-                
-                // Copying elements
-                std::copy(i.begin(), i.end(), arma::access::rwp(res.row_indices));
-                std::copy(p.begin(), p.end(), arma::access::rwp(res.col_ptrs));
-                std::copy(x.begin(), x.end(), arma::access::rwp(res.values));
+                res = arma::sp_mat(true, arma::join_cols(ti, tj), tx, nrow, ncol, true, false);
             }
             else if (type == "dtTMatrix" || mat.is("dtTMatrix")) {
-                IntegerVector ti = mat.slot("i");
-                IntegerVector tj = mat.slot("j");
-                Vector<RTYPE> tx = mat.slot("x");
-                std::string diag = Rcpp::as<std::string>(mat.slot("diag"));
+                arma::urowvec ti = mat.slot("i");
+                arma::urowvec tj = mat.slot("j");
+                arma::Col<T> tx = mat.slot("x");
                 
-                // Create a map to sort the i, j, x
-                typedef std::pair<int, int> Key;
-                typedef std::map<Key, double> Map;
-                Map jix;
-                int tnnz = tx.size();
-                for(int idx = 0; idx < tnnz; idx++){
-                    Key p(tj[idx], ti[idx]);
-                    if (jix.find(p) != jix.end()) {
-                        jix[p] += tx[idx];
-                    } else {
-                        jix[p] = tx[idx];
-                    }
-                }
-                
-                // Get the new i, j, x;
-                int nnz = jix.size();
-                std::vector<int> i;
-                std::vector<int> j;
-                std::vector<double> x;
-                IntegerVector p = IntegerVector(ncol + 1);
-                
-                for(Map::iterator tmp = jix.begin(); tmp != jix.end(); tmp++){
-                    j.push_back((tmp -> first).first);
-                    i.push_back((tmp -> first).second);
-                    x.push_back(tmp -> second);
-                }
-                
-                // Count the number of nnz in each column
-                for(int idx = 0; idx < nnz; idx++){
-                    int col = j[idx];
-                    p[col + 1]++;
-                }
-                
-                // Cumsum p
-                for(int col = 0, cumsum = 0; col < ncol + 1; col++){
-                    cumsum += p[col];
-                    p[col] = cumsum;
-                }
-                
-                // Making space for the elements
-                res.mem_resize(static_cast<unsigned>(x.size()));
-                
-                // Copying elements
-                std::copy(i.begin(), i.end(), arma::access::rwp(res.row_indices));
-                std::copy(p.begin(), p.end(), arma::access::rwp(res.col_ptrs));
-                std::copy(x.begin(), x.end(), arma::access::rwp(res.values));
-                
-                if (diag == "U"){
+                res = arma::sp_mat(true, arma::join_cols(ti, tj), tx, nrow, ncol, true, false);
+                if (Rcpp::as<std::string>(mat.slot("diag")) == "U") {
                     res.diag().ones();
                 }
             }
             else if (type == "dsTMatrix" || mat.is("dsTMatrix")) {
-                IntegerVector ti = mat.slot("i");
-                IntegerVector tj = mat.slot("j");
-                Vector<RTYPE> tx = mat.slot("x");
-                std::string uplo = Rcpp::as<std::string>(mat.slot("uplo"));
-                
-                // Create a map to sort the i, j, x
-                typedef std::pair<int, int> Key;
-                typedef std::map<Key, double> Map;
-                Map jix;
-                int tnnz = tx.size();
-                for(int idx = 0; idx < tnnz; idx++){
-                    Key p(tj[idx], ti[idx]);
-                    if (jix.find(p) != jix.end()) {
-                        jix[p] += tx[idx];
-                    } else {
-                        jix[p] = tx[idx];
-                    }
-                }
-                
-                // Get the new i, j, x;
-                int nnz = jix.size();
-                std::vector<int> i;
-                std::vector<int> j;
-                std::vector<double> x;
-                IntegerVector p = IntegerVector(ncol + 1);
-                
-                for(Map::iterator tmp = jix.begin(); tmp != jix.end(); tmp++){
-                    j.push_back((tmp -> first).first);
-                    i.push_back((tmp -> first).second);
-                    x.push_back(tmp -> second);
-                }
-                
-                // Count the number of nnz in each column
-                for(int idx = 0; idx < nnz; idx++){
-                    int col = j[idx];
-                    p[col + 1]++;
-                }
-                
-                // Cumsum p
-                for(int col = 0, cumsum = 0; col < ncol + 1; col++){
-                    cumsum += p[col];
-                    p[col] = cumsum;
-                }
-                
-                // Making space for the elements
-                res.mem_resize(static_cast<unsigned>(x.size()));
-                
-                // Copying elements
-                std::copy(i.begin(), i.end(), arma::access::rwp(res.row_indices));
-                std::copy(p.begin(), p.end(), arma::access::rwp(res.col_ptrs));
-                std::copy(x.begin(), x.end(), arma::access::rwp(res.values));
-              
-                if (uplo == "U") {
-                    res = symmatu(res);
-                } else {
-                    res = symmatl(res);
-                }
+                arma::urowvec ti = mat.slot("i");
+                arma::urowvec tj = mat.slot("j");
+                arma::Col<T> tx = mat.slot("x");
+
+                res = arma::sp_mat(true, arma::join_cols(ti, tj), tx, nrow, ncol, true, false);
+                res = Rcpp::as<std::string>(mat.slot("uplo")) == "U" ? symmatu(res) : symmatl(res);
             }
             else if (type == "dgRMatrix" || mat.is("dgRMatrix")) {
                 IntegerVector rj = mat.slot("j");
@@ -361,6 +227,9 @@ namespace traits {
                 // Making space for the elements
                 res.mem_resize(static_cast<unsigned>(x.size()));
                 
+                // In order to access the internal arrays of the SpMat class
+                res.sync();
+                
                 // Copying elements
                 std::copy(i.begin(), i.end(), arma::access::rwp(res.row_indices));
                 std::copy(p.begin(), p.end(), arma::access::rwp(res.col_ptrs));
@@ -412,6 +281,9 @@ namespace traits {
                 // Making space for the elements
                 res.mem_resize(static_cast<unsigned>(x.size()));
                 
+                // In order to access the internal arrays of the SpMat class
+                res.sync();
+                
                 // Copying elements
                 std::copy(i.begin(), i.end(), arma::access::rwp(res.row_indices));
                 std::copy(p.begin(), p.end(), arma::access::rwp(res.col_ptrs));
@@ -467,6 +339,9 @@ namespace traits {
                 // Making space for the elements
                 res.mem_resize(static_cast<unsigned>(x.size()));
                 
+                // In order to access the internal arrays of the SpMat class
+                res.sync();
+                
                 // Copying elements
                 std::copy(i.begin(), i.end(), arma::access::rwp(res.row_indices));
                 std::copy(p.begin(), p.end(), arma::access::rwp(res.col_ptrs));
@@ -514,6 +389,9 @@ namespace traits {
                 // Making space for the elements
                 res.mem_resize(static_cast<unsigned>(x.size()));
                 
+                // In order to access the internal arrays of the SpMat class
+                res.sync();
+                
                 // Copying elements
                 std::copy(i.begin(), i.end(), arma::access::rwp(res.row_indices));
                 std::copy(p.begin(), p.end(), arma::access::rwp(res.col_ptrs));
@@ -545,6 +423,9 @@ namespace traits {
                 // Making space for the elements
                 res.mem_resize(static_cast<unsigned>(x.size()));
               
+                // In order to access the internal arrays of the SpMat class
+                res.sync();
+                
                 // Copying elements
                 std::copy(i.begin(), i.end(), arma::access::rwp(res.row_indices));
                 std::copy(p.begin(), p.end(), arma::access::rwp(res.col_ptrs));
@@ -580,6 +461,9 @@ namespace traits {
                 // Making space for the elements
                 res.mem_resize(static_cast<unsigned>(x.size()));
 
+                // In order to access the internal arrays of the SpMat class
+                res.sync();
+
                 // Copying elements
                 std::copy(i.begin(), i.end(), arma::access::rwp(res.row_indices));
                 std::copy(p.begin(), p.end(), arma::access::rwp(res.col_ptrs));
@@ -589,6 +473,9 @@ namespace traits {
                 Rcpp::stop(type + " is not supported.");
             }
             
+            // In order to access the internal arrays of the SpMat class
+            res.sync();
+
             // Setting the sentinel
             arma::access::rw(res.col_ptrs[static_cast<unsigned>(ncol + 1)]) =
               std::numeric_limits<arma::uword>::max();
