@@ -50,6 +50,11 @@ template<typename eT> class SpSubview;
 template<typename eT> class diagview;
 template<typename eT> class spdiagview;
 
+template<typename eT> class MapMat;
+template<typename eT> class MapMat_val;
+template<typename eT> class MapMat_elem;
+template<typename eT> class MapMat_svel;
+
 template<typename eT, typename T1>              class subview_elem1;
 template<typename eT, typename T1, typename T2> class subview_elem2;
 
@@ -267,6 +272,24 @@ enum file_type
   };
 
 
+struct hdf5_name
+  {
+  const std::string filename;
+  const std::string dsname;
+  
+  inline
+  hdf5_name(const std::string& in_filename)
+    : filename(in_filename)
+    {}
+  
+  inline
+  hdf5_name(const std::string& in_filename, const std::string& in_dsname)
+    : filename(in_filename)
+    , dsname  (in_dsname  )
+    {}
+  };
+
+
 //! @}
 
 
